/*
 * Copyright (c) 2021 D3 Engineering, LLC.
 * GPL-v2 License
 * Adapted by: Luis Leon <luis.leon@ridgerun.com>
 */

#pragma once

#define CU_CHECK_FL(STMT, FN, LN, ACTION)                    \
  do {                                                       \
    CUresult status = (STMT);                                \
    if (status != CUDA_SUCCESS) {                            \
      const char *name;                                      \
      const char *desc;                                      \
      std::string errmsg;                                    \
      if (CUDA_SUCCESS == cuGetErrorName(status, &name)) {   \
        errmsg += name;                                      \
      } else {                                               \
        errmsg += "Unknown error";                           \
      }                                                      \
      errmsg += " (";                                        \
      if (CUDA_SUCCESS == cuGetErrorString(status, &desc)) { \
        errmsg += desc;                                      \
      } else {                                               \
        errmsg += "no details";                              \
      }                                                      \
      errmsg += ") at ";                                     \
      errmsg += (FN);                                        \
      errmsg += ':';                                         \
      errmsg += std::to_string(LN);                          \
      errmsg += " (`";                                       \
      errmsg += (#STMT);                                     \
      errmsg += "`)";                                        \
      ACTION;                                                \
    }                                                        \
  } while (0)

#define CUDA_CHECK_FL(STMT, FN, LN, ACTION)             \
  do {                                                  \
    cudaError_t cuda_err = (STMT);                      \
    if (cuda_err != cudaSuccess) {                      \
      std::string errmsg(cudaGetErrorString(cuda_err)); \
      errmsg += " at ";                                 \
      errmsg += (FN);                                   \
      errmsg += ':';                                    \
      errmsg += std::to_string(LN);                     \
      errmsg += " (`";                                  \
      errmsg += (#STMT);                                \
      errmsg += "`)";                                   \
      ACTION;                                           \
    }                                                   \
  } while (0)

#define CU_JUMP_IF_FAIL(LABEL, STMT)                              \
  CU_CHECK_FL(STMT, __FILE__, __LINE__,                           \
              GST_ERROR("CUDA driver ERROR: %s", errmsg.c_str()); \
              goto LABEL)

#define CUDA_JUMP_IF_FAIL(LABEL, STMT)                               \
  CUDA_CHECK_FL(STMT, __FILE__, __LINE__,                            \
                GST_ERROR("CUDA runtime ERROR: %s", errmsg.c_str()); \
                goto LABEL)
